
#ifndef LUBISHEV_SCENE_H
#define LUBISHEV_SCENE_H

#include <lubishev/Common.h>
#include <lubishev/DB.h>
#include <lubishev/Edit.h>
#include <lubishev/SceneEvent.h>

#include <QCalendarWidget>
#include <QEvent>
#include <QKeyEvent>
#include <QMouseEvent>
#include <QWidget>

namespace lubishev {

class Scene: public QWidget {
    Q_OBJECT
    public:
        Scene(DB *db, QWidget *parent = 0);
        ~Scene();

        bool calendarIsVisible() const;
        void clearEvents();
        float secondHeight() const;
        void setCalendarVisible(bool state);
        void setInterval(time_t start, time_t end);
        void setInterval(const QDate &date);
        void setIntervalNow();
        u16 snap() const;
        float timeToY(time_t tm);

    public slots:
        void onCalendarClick(const QDate &date);

    signals:
        void cursorTimeChanged(time_t cursor);

    protected:
        void keyPressEvent(QKeyEvent *e);
        void keyReleaseEvent(QKeyEvent *e);
        void mouseDoubleClickEvent(QMouseEvent *e);
        void mouseMoveEvent(QMouseEvent *e);
        void mousePressEvent(QMouseEvent *e);
        void mouseReleaseEvent(QMouseEvent *e);
        void paintEvent(QPaintEvent *e);
        void resizeEvent(QResizeEvent *e);
        void wheelEvent(QWheelEvent *e);

    private:
        // Types.
        typedef std::vector<Event*> Events;
        typedef Events::iterator EIt;

        typedef std::vector<SceneEvent*> SceneEvents;
        typedef SceneEvents::iterator SEIt;
        typedef SceneEvents::const_iterator SECIt;

        typedef std::map<Event*, SceneEvent*> ESE;
        typedef ESE::iterator ESEIt;
        typedef ESE::const_iterator ESECIt;

        typedef std::vector<time_t> VisibleHours;
        typedef VisibleHours::iterator VHIt;

        void createEvent(time_t start, time_t end);
        void doEventDiff(SceneEvent *se, time_t diff);
        bool hoverEvent(bool next = false);
        // Returns 0 when no next hour.
        time_t nextHour(time_t start, time_t end);
        void refreshCell();
        void refreshEvents();
        void refreshInterval();
        bool refreshTimeLine(const QPointF &pos);
        bool sceneEventsEqual(const SceneEvents &se1, const SceneEvents &se2);
        time_t snapTime(time_t t);
        void tryToSelectEvent();
        SceneEvents xyToEvents(const QPointF &pos);
        inline time_t yToTime(float y);

        QPen mPen;
        QFont mFont;
        // Focused time interval.
        time_t mStart;
        time_t mEnd;
        float mSecondHeight;
        VisibleHours mVisibleHours;
        time_t mEventStart;
        time_t mEventEnd;
        u64 mEventID;
        SceneEvents mLastHovered;
        u32 mLastHoveredEventID;
        time_t mCursorTime;
        u16 mSnap;
        bool mZoom;
        s32 mZoomInterval;
        ESE mESE;
        time_t mClickStart;
        bool mClick;
        Edit *mEdit;
        DB *mDB;
        QCalendarWidget *mCalendar;
};

} // namespace lubishev

#endif // LUBISHEV_SCENE_H

